@@ -705,14 +705,44 @@ action_new_window (GSimpleAction *action,
                    GVariant      *parameter,
                    gpointer       user_data)
 {
-    GtkApplication *application = user_data;
-    GFile *home;
+    NautilusApplication *application;
+    g_autoptr (GFile) home = NULL;
 
+    application = NAUTILUS_APPLICATION (user_data);
     home = g_file_new_for_path (g_get_home_dir ());
-    nautilus_application_open_location_full (NAUTILUS_APPLICATION (application), home,
-                                             NAUTILUS_WINDOW_OPEN_FLAG_NEW_WINDOW, NULL, NULL, NULL);
 
-    g_object_unref (home);
+    nautilus_application_open_location_full (application, home,
+                                             NAUTILUS_WINDOW_OPEN_FLAG_NEW_WINDOW,
+                                             NULL, NULL, NULL);
+}
+
+static void
+action_clone_window (GSimpleAction *action,
+                     GVariant      *parameter,
+                     gpointer       user_data)
+{
+    NautilusWindowSlot *active_slot = NULL;
+    NautilusWindow *active_window = NULL;
+    GtkApplication *application = user_data;
+    g_autoptr (GFile) location = NULL;
+    NautilusView *current_view;
+
+    active_window = NAUTILUS_WINDOW (gtk_application_get_active_window (application));
+    active_slot = nautilus_window_get_active_slot (active_window);
+    current_view = nautilus_window_slot_get_current_view (active_slot);
+
+    if (current_view != NULL &&
+        nautilus_view_is_searching (current_view))
+    {
+        location = g_file_new_for_path (g_get_home_dir ());
+    }
+    else
+    {
+        location = nautilus_window_slot_get_location (active_slot);
+    }
+
+    nautilus_application_open_location_full (NAUTILUS_APPLICATION (application), location,
+                                             NAUTILUS_WINDOW_OPEN_FLAG_NEW_WINDOW, NULL, NULL, NULL);
 }
 
 static void
@@ -838,6 +868,7 @@ action_show_help_overlay (GSimpleAction *action,
 static GActionEntry app_entries[] =
 {
     { "new-window", action_new_window, NULL, NULL, NULL },
+    { "clone-window", action_clone_window, NULL, NULL, NULL },
     { "preferences", action_preferences, NULL, NULL, NULL },
     { "show-hide-sidebar", NULL, NULL, "true", action_show_hide_sidebar },
     { "about", action_about, NULL, NULL, NULL },
@@ -900,7 +931,7 @@ const GOptionEntry options[] =
       N_("Quit Nautilus."), NULL },
     { "select", 's', 0, G_OPTION_ARG_NONE, NULL,
       N_("Select specified URI in parent folder."), NULL },
-    { G_OPTION_REMAINING, 0, 0, G_OPTION_ARG_STRING_ARRAY, NULL, NULL, N_("[URI...]") },
+    { G_OPTION_REMAINING, 0, 0, G_OPTION_ARG_STRING_ARRAY, NULL, NULL, N_("[URI…]") },
 
     { NULL }
 };
@@ -1068,7 +1099,7 @@ theme_changed (GtkSettings *settings)
     screen = gdk_screen_get_default ();
 
     /* CSS that themes can override */
-    if (g_str_equal (theme, "Adwaita") || g_str_equal (theme, "Adwaita-dark"))
+    if (g_str_equal (theme, "Adwaita"))
     {
         if (provider == NULL)
         {
@@ -1200,7 +1231,7 @@ nautilus_application_startup_common (NautilusApplication *self)
      */
     G_APPLICATION_CLASS (nautilus_application_parent_class)->startup (G_APPLICATION (self));
 
-    gtk_window_set_default_icon_name ("system-file-manager");
+    gtk_window_set_default_icon_name ("org.gnome.Nautilus");
 
     setup_theme_extensions ();
 
