@@ -27,6 +27,7 @@
 #include "nautilus-error-reporting.h"
 #include "nautilus-files-view-dnd.h"
 #include "nautilus-toolbar.h"
+#include "nautilus-view.h"
 
 #include <stdlib.h>
 #include <errno.h>
@@ -85,7 +86,7 @@ typedef enum
     MENU_ITEM_TYPE_TREE
 } MenuItemType;
 
-struct NautilusCanvasViewDetails
+typedef struct
 {
     GList *icons_not_positioned;
 
@@ -100,15 +101,13 @@ struct NautilusCanvasViewDetails
     gboolean supports_scaling;
     gboolean supports_keep_aligned;
 
-    /* Needed for async operations. Suposedly we would use cancellable and gtask,
+    /* FIXME: Needed for async operations. Suposedly we would use cancellable and gtask,
      * sadly gtkclipboard doesn't support that.
      * We follow this pattern for checking validity of the object in the views.
      * Ideally we would connect to a weak reference and do a cancellable.
      */
     gboolean destroyed;
-
-    GIcon *icon;
-};
+} NautilusCanvasViewPrivate;
 
 /* Note that the first item in this list is the default sort,
  * and that the items show up in the menu in the order they
@@ -117,69 +116,71 @@ struct NautilusCanvasViewDetails
 static const SortCriterion sort_criteria[] =
 {
     {
-     NAUTILUS_FILE_SORT_BY_DISPLAY_NAME,
-     "name",
-     "name",
-     FALSE
-},
+        NAUTILUS_FILE_SORT_BY_DISPLAY_NAME,
+        "name",
+        "name",
+        FALSE
+    },
     {
-     NAUTILUS_FILE_SORT_BY_DISPLAY_NAME,
-     "name",
-     "name-desc",
-     TRUE
-},
+        NAUTILUS_FILE_SORT_BY_DISPLAY_NAME,
+        "name",
+        "name-desc",
+        TRUE
+    },
     {
-     NAUTILUS_FILE_SORT_BY_SIZE,
-     "size",
-     "size",
-     TRUE
-},
+        NAUTILUS_FILE_SORT_BY_SIZE,
+        "size",
+        "size",
+        TRUE
+    },
     {
-     NAUTILUS_FILE_SORT_BY_TYPE,
-     "type",
-     "type",
-     FALSE
-},
+        NAUTILUS_FILE_SORT_BY_TYPE,
+        "type",
+        "type",
+        FALSE
+    },
     {
-     NAUTILUS_FILE_SORT_BY_MTIME,
-     "modification date",
-     "modification-date",
-     FALSE
-},
+        NAUTILUS_FILE_SORT_BY_MTIME,
+        "modification date",
+        "modification-date",
+        FALSE
+    },
     {
-     NAUTILUS_FILE_SORT_BY_MTIME,
-     "modification date",
-     "modification-date-desc",
-     TRUE
-},
+        NAUTILUS_FILE_SORT_BY_MTIME,
+        "modification date",
+        "modification-date-desc",
+        TRUE
+    },
     {
-     NAUTILUS_FILE_SORT_BY_ATIME,
-     "access date",
-     "access-date",
-     FALSE
-},
+        NAUTILUS_FILE_SORT_BY_ATIME,
+        "access date",
+        "access-date",
+        FALSE
+    },
     {
-     NAUTILUS_FILE_SORT_BY_ATIME,
-     "access date",
-     "access-date-desc",
-     TRUE
-},
+        NAUTILUS_FILE_SORT_BY_ATIME,
+        "access date",
+        "access-date-desc",
+        TRUE
+    },
     {
-     NAUTILUS_FILE_SORT_BY_TRASHED_TIME,
-     "trashed",
-     "trash-time",
-     TRUE,
-     nautilus_file_is_in_trash
-},
+        NAUTILUS_FILE_SORT_BY_TRASHED_TIME,
+        "trashed",
+        "trash-time",
+        TRUE,
+        nautilus_file_is_in_trash
+    },
     {
-     NAUTILUS_FILE_SORT_BY_SEARCH_RELEVANCE,
-     NULL,
-     "search-relevance",
-     TRUE,
-     nautilus_file_is_in_search
-}
+        NAUTILUS_FILE_SORT_BY_SEARCH_RELEVANCE,
+        NULL,
+        "search-relevance",
+        TRUE,
+        nautilus_file_is_in_search
+    }
 };
 
+G_DEFINE_TYPE_WITH_PRIVATE (NautilusCanvasView, nautilus_canvas_view, NAUTILUS_TYPE_FILES_VIEW);
+
 static void                 nautilus_canvas_view_set_directory_sort_by (NautilusCanvasView  *canvas_view,
                                                                         NautilusFile        *file,
                                                                         const SortCriterion *sort);
@@ -197,22 +198,22 @@ static void on_clipboard_owner_changed (GtkClipboard *clipboard,
                                         GdkEvent     *event,
                                         gpointer      user_data);
 
-G_DEFINE_TYPE (NautilusCanvasView, nautilus_canvas_view, NAUTILUS_TYPE_FILES_VIEW);
-
 static void
 nautilus_canvas_view_destroy (GtkWidget *object)
 {
     NautilusCanvasView *canvas_view;
+    NautilusCanvasViewPrivate *priv;
     GtkClipboard *clipboard;
 
     canvas_view = NAUTILUS_CANVAS_VIEW (object);
+    priv = nautilus_canvas_view_get_instance_private (canvas_view);
 
     nautilus_canvas_view_clear (NAUTILUS_FILES_VIEW (object));
 
-    if (canvas_view->details->react_to_canvas_change_idle_id != 0)
+    if (priv->react_to_canvas_change_idle_id != 0)
     {
-        g_source_remove (canvas_view->details->react_to_canvas_change_idle_id);
-        canvas_view->details->react_to_canvas_change_idle_id = 0;
+        g_source_remove (priv->react_to_canvas_change_idle_id);
+        priv->react_to_canvas_change_idle_id = 0;
     }
 
     clipboard = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
@@ -220,10 +221,10 @@ nautilus_canvas_view_destroy (GtkWidget *object)
                                           on_clipboard_owner_changed,
                                           canvas_view);
 
-    if (canvas_view->details->icons_not_positioned)
+    if (priv->icons_not_positioned)
     {
-        nautilus_file_list_free (canvas_view->details->icons_not_positioned);
-        canvas_view->details->icons_not_positioned = NULL;
+        nautilus_file_list_free (priv->icons_not_positioned);
+        priv->icons_not_positioned = NULL;
     }
 
     GTK_WIDGET_CLASS (nautilus_canvas_view_parent_class)->destroy (object);
@@ -232,7 +233,11 @@ nautilus_canvas_view_destroy (GtkWidget *object)
 static NautilusCanvasContainer *
 get_canvas_container (NautilusCanvasView *canvas_view)
 {
-    return NAUTILUS_CANVAS_CONTAINER (canvas_view->details->canvas_container);
+    NautilusCanvasViewPrivate *priv;
+
+    priv = nautilus_canvas_view_get_instance_private (canvas_view);
+
+    return NAUTILUS_CANVAS_CONTAINER (priv->canvas_container);
 }
 
 NautilusCanvasContainer *
@@ -244,9 +249,13 @@ nautilus_canvas_view_get_canvas_container (NautilusCanvasView *canvas_view)
 static gboolean
 nautilus_canvas_view_supports_manual_layout (NautilusCanvasView *view)
 {
+    NautilusCanvasViewPrivate *priv;
+
     g_return_val_if_fail (NAUTILUS_IS_CANVAS_VIEW (view), FALSE);
 
-    return view->details->supports_manual_layout;
+    priv = nautilus_canvas_view_get_instance_private (view);
+
+    return priv->supports_manual_layout;
 }
 
 static gboolean
@@ -307,9 +316,11 @@ update_sort_criterion (NautilusCanvasView  *canvas_view,
                        gboolean             set_metadata)
 {
     NautilusFile *file;
+    NautilusCanvasViewPrivate *priv;
     const SortCriterion *overrided_sort_criterion;
 
     file = nautilus_files_view_get_directory_as_file (NAUTILUS_FILES_VIEW (canvas_view));
+    priv = nautilus_canvas_view_get_instance_private (canvas_view);
 
     /* Make sure we use the default one and not one that the user used previously
      * of the change to not allow sorting on search and recent, or the
@@ -318,7 +329,7 @@ update_sort_criterion (NautilusCanvasView  *canvas_view,
     {
         overrided_sort_criterion = get_default_sort_order (file);
     }
-    else if (sort != NULL && canvas_view->details->sort != sort)
+    else if (sort != NULL && priv->sort != sort)
     {
         overrided_sort_criterion = sort;
         if (set_metadata)
@@ -334,7 +345,7 @@ update_sort_criterion (NautilusCanvasView  *canvas_view,
         return;
     }
 
-    canvas_view->details->sort = overrided_sort_criterion;
+    priv->sort = overrided_sort_criterion;
 }
 
 void
@@ -436,12 +447,13 @@ nautilus_canvas_view_remove_file (NautilusFilesView *view,
 }
 
 static void
-nautilus_canvas_view_add_file (NautilusFilesView *view,
-                               NautilusFile      *file,
-                               NautilusDirectory *directory)
+nautilus_canvas_view_add_files (NautilusFilesView *view,
+                                GList             *files,
+                                NautilusDirectory *directory)
 {
     NautilusCanvasView *canvas_view;
     NautilusCanvasContainer *canvas_container;
+    GList *l;
 
     g_assert (directory == nautilus_files_view_get_model (view));
 
@@ -454,10 +466,13 @@ nautilus_canvas_view_add_file (NautilusFilesView *view,
         nautilus_canvas_container_reset_scroll_region (canvas_container);
     }
 
-    if (nautilus_canvas_container_add (canvas_container,
-                                       NAUTILUS_CANVAS_ICON_DATA (file)))
+    for (l = files; l != NULL; l = l->next)
     {
-        nautilus_file_ref (file);
+        if (nautilus_canvas_container_add (canvas_container,
+                                           NAUTILUS_CANVAS_ICON_DATA (l->data)))
+        {
+            nautilus_file_ref (NAUTILUS_FILE (l->data));
+        }
     }
 }
 
@@ -481,25 +496,37 @@ nautilus_canvas_view_file_changed (NautilusFilesView *view,
 static gboolean
 nautilus_canvas_view_supports_auto_layout (NautilusCanvasView *view)
 {
+    NautilusCanvasViewPrivate *priv;
+
     g_return_val_if_fail (NAUTILUS_IS_CANVAS_VIEW (view), FALSE);
 
-    return view->details->supports_auto_layout;
+    priv = nautilus_canvas_view_get_instance_private (view);
+
+    return priv->supports_auto_layout;
 }
 
 static gboolean
 nautilus_canvas_view_supports_scaling (NautilusCanvasView *view)
 {
+    NautilusCanvasViewPrivate *priv;
+
     g_return_val_if_fail (NAUTILUS_IS_CANVAS_VIEW (view), FALSE);
 
-    return view->details->supports_scaling;
+    priv = nautilus_canvas_view_get_instance_private (view);
+
+    return priv->supports_scaling;
 }
 
 static gboolean
 nautilus_canvas_view_supports_keep_aligned (NautilusCanvasView *view)
 {
+    NautilusCanvasViewPrivate *priv;
+
     g_return_val_if_fail (NAUTILUS_IS_CANVAS_VIEW (view), FALSE);
 
-    return view->details->supports_keep_aligned;
+    priv = nautilus_canvas_view_get_instance_private (view);
+
+    return priv->supports_keep_aligned;
 }
 
 static const SortCriterion *
@@ -764,9 +791,12 @@ on_clipboard_contents_received (GtkClipboard     *clipboard,
                                 GtkSelectionData *selection_data,
                                 gpointer          user_data)
 {
+    NautilusCanvasViewPrivate *priv;
     NautilusCanvasView *view = NAUTILUS_CANVAS_VIEW (user_data);
 
-    if (view->details->destroyed)
+    priv = nautilus_canvas_view_get_instance_private (view);
+
+    if (priv->destroyed)
     {
         /* We've been destroyed since call */
         g_object_unref (view);
@@ -798,7 +828,6 @@ on_clipboard_contents_received (GtkClipboard     *clipboard,
 static void
 update_clipboard_status (NautilusCanvasView *view)
 {
-
     g_object_ref (view);     /* Need to keep the object alive until we get the reply */
     gtk_clipboard_request_contents (nautilus_clipboard_get (GTK_WIDGET (view)),
                                     nautilus_clipboard_get_atom (),
@@ -829,7 +858,7 @@ nautilus_canvas_view_end_loading (NautilusFilesView *view,
 static NautilusCanvasZoomLevel
 nautilus_canvas_view_get_zoom_level (NautilusFilesView *view)
 {
-    g_return_val_if_fail (NAUTILUS_IS_CANVAS_VIEW (view), NAUTILUS_CANVAS_ZOOM_LEVEL_STANDARD);
+    g_return_val_if_fail (NAUTILUS_IS_CANVAS_VIEW (view), NAUTILUS_CANVAS_ZOOM_LEVEL_LARGE);
 
     return nautilus_canvas_container_get_zoom_level (get_canvas_container (NAUTILUS_CANVAS_VIEW (view)));
 }
@@ -1019,13 +1048,17 @@ action_zoom_to_level (GSimpleAction *action,
 static void
 switch_to_manual_layout (NautilusCanvasView *canvas_view)
 {
+    NautilusCanvasViewPrivate *priv;
+
+    priv = nautilus_canvas_view_get_instance_private (canvas_view);
+
     if (!nautilus_canvas_view_using_auto_layout (canvas_view) ||
         !nautilus_files_view_is_editable (NAUTILUS_FILES_VIEW (canvas_view)))
     {
         return;
     }
 
-    canvas_view->details->sort = &sort_criteria[0];
+    priv->sort = &sort_criteria[0];
 
     nautilus_canvas_container_set_auto_layout
         (get_canvas_container (canvas_view), FALSE);
@@ -1122,8 +1155,10 @@ nautilus_canvas_view_update_actions_state (NautilusFilesView *view)
     GAction *action;
     gboolean keep_aligned;
     NautilusCanvasView *canvas_view;
+    NautilusCanvasViewPrivate *priv;
 
     canvas_view = NAUTILUS_CANVAS_VIEW (view);
+    priv = nautilus_canvas_view_get_instance_private (canvas_view);
 
     NAUTILUS_FILES_VIEW_CLASS (nautilus_canvas_view_parent_class)->update_actions_state (view);
 
@@ -1143,11 +1178,11 @@ nautilus_canvas_view_update_actions_state (NautilusFilesView *view)
         sort_state = g_action_group_get_action_state (view_action_group, "sort");
 
         if (g_strcmp0 (g_variant_get_string (sort_state, NULL),
-                       NAUTILUS_CANVAS_VIEW (view)->details->sort->action_target_name) != 0)
+                       priv->sort->action_target_name) != 0)
         {
             g_action_group_change_action_state (view_action_group,
                                                 "sort",
-                                                g_variant_new_string (NAUTILUS_CANVAS_VIEW (view)->details->sort->action_target_name));
+                                                g_variant_new_string (priv->sort->action_target_name));
         }
 
         g_variant_unref (sort_state);
@@ -1159,8 +1194,8 @@ nautilus_canvas_view_update_actions_state (NautilusFilesView *view)
                                  !showing_search_directory (view));
     action = g_action_map_lookup_action (G_ACTION_MAP (view_action_group), "keep-aligned");
     g_simple_action_set_enabled (G_SIMPLE_ACTION (action),
-                                 canvas_view->details->supports_keep_aligned);
-    if (canvas_view->details->supports_keep_aligned)
+                                 priv->supports_keep_aligned);
+    if (priv->supports_keep_aligned)
     {
         keep_aligned = nautilus_canvas_container_is_keep_aligned (get_canvas_container (canvas_view));
         g_action_change_state (action, g_variant_new_boolean (keep_aligned));
@@ -1400,11 +1435,15 @@ nautilus_canvas_view_compare_files (NautilusCanvasView *canvas_view,
                                     NautilusFile       *a,
                                     NautilusFile       *b)
 {
+    NautilusCanvasViewPrivate *priv;
+
+    priv = nautilus_canvas_view_get_instance_private (canvas_view);
+
     return nautilus_file_compare_for_sort
-               (a, b, canvas_view->details->sort->sort_type,
+               (a, b, priv->sort->sort_type,
                /* Use type-unsafe cast for performance */
                nautilus_files_view_should_sort_directories_first ((NautilusFilesView *) canvas_view),
-               canvas_view->details->sort->reverse_order);
+               priv->sort->reverse_order);
 }
 
 static int
@@ -1707,11 +1746,14 @@ initialize_canvas_container (NautilusCanvasView      *canvas_view,
                              NautilusCanvasContainer *canvas_container)
 {
     GtkWidget *content_widget;
+    NautilusCanvasViewPrivate *priv;
+
+    priv = nautilus_canvas_view_get_instance_private (canvas_view);
 
     content_widget = nautilus_files_view_get_content_widget (NAUTILUS_FILES_VIEW (canvas_view));
-    canvas_view->details->canvas_container = GTK_WIDGET (canvas_container);
+    priv->canvas_container = GTK_WIDGET (canvas_container);
     g_object_add_weak_pointer (G_OBJECT (canvas_container),
-                               (gpointer *) &canvas_view->details->canvas_container);
+                               (gpointer *) &priv->canvas_container);
 
     gtk_widget_set_can_focus (GTK_WIDGET (canvas_container), TRUE);
 
@@ -1889,64 +1931,54 @@ nautilus_canvas_view_set_property (GObject      *object,
                                    GParamSpec   *pspec)
 {
     NautilusCanvasView *canvas_view;
+    NautilusCanvasViewPrivate *priv;
 
     canvas_view = NAUTILUS_CANVAS_VIEW (object);
+    priv = nautilus_canvas_view_get_instance_private (canvas_view);
 
     switch (prop_id)
     {
         case PROP_SUPPORTS_AUTO_LAYOUT:
-            {
-                canvas_view->details->supports_auto_layout = g_value_get_boolean (value);
-            }
-            break;
+        {
+            priv->supports_auto_layout = g_value_get_boolean (value);
+        }
+        break;
 
         case PROP_SUPPORTS_MANUAL_LAYOUT:
-            {
-                canvas_view->details->supports_manual_layout = g_value_get_boolean (value);
-            }
-            break;
+        {
+            priv->supports_manual_layout = g_value_get_boolean (value);
+        }
+        break;
 
         case PROP_SUPPORTS_SCALING:
-            {
-                canvas_view->details->supports_scaling = g_value_get_boolean (value);
-            }
-            break;
+        {
+            priv->supports_scaling = g_value_get_boolean (value);
+        }
+        break;
 
         case PROP_SUPPORTS_KEEP_ALIGNED:
-            {
-                canvas_view->details->supports_keep_aligned = g_value_get_boolean (value);
-            }
-            break;
+        {
+            priv->supports_keep_aligned = g_value_get_boolean (value);
+        }
+        break;
 
         default:
-            {
-                G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-            }
-            break;
+        {
+            G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+        }
+        break;
     }
 }
 
-static void
-nautilus_canvas_view_finalize (GObject *object)
-{
-    NautilusCanvasView *canvas_view;
-
-    canvas_view = NAUTILUS_CANVAS_VIEW (object);
-
-    g_clear_object (&canvas_view->details->icon);
-
-    g_free (canvas_view->details);
-
-    G_OBJECT_CLASS (nautilus_canvas_view_parent_class)->finalize (object);
-}
-
 static void
 nautilus_canvas_view_dispose (GObject *object)
 {
     NautilusCanvasView *canvas_view;
+    NautilusCanvasViewPrivate *priv;
 
     canvas_view = NAUTILUS_CANVAS_VIEW (object);
-    canvas_view->details->destroyed = TRUE;
+    priv = nautilus_canvas_view_get_instance_private (canvas_view);
+    priv->destroyed = TRUE;
 
     g_signal_handlers_disconnect_by_func (nautilus_preferences,
                                           default_sort_order_changed_callback,
@@ -1963,14 +1995,6 @@ nautilus_canvas_view_dispose (GObject *object)
     G_OBJECT_CLASS (nautilus_canvas_view_parent_class)->dispose (object);
 }
 
-static GIcon *
-nautilus_canvas_view_get_icon (NautilusFilesView *view)
-{
-    g_return_val_if_fail (NAUTILUS_IS_CANVAS_VIEW (view), NULL);
-
-    return NAUTILUS_CANVAS_VIEW (view)->details->icon;
-}
-
 static void
 nautilus_canvas_view_class_init (NautilusCanvasViewClass *klass)
 {
@@ -1981,14 +2005,13 @@ nautilus_canvas_view_class_init (NautilusCanvasViewClass *klass)
     oclass = G_OBJECT_CLASS (klass);
 
     oclass->set_property = nautilus_canvas_view_set_property;
-    oclass->finalize = nautilus_canvas_view_finalize;
     oclass->dispose = nautilus_canvas_view_dispose;
 
     GTK_WIDGET_CLASS (klass)->destroy = nautilus_canvas_view_destroy;
 
     klass->create_canvas_container = real_create_canvas_container;
 
-    nautilus_files_view_class->add_file = nautilus_canvas_view_add_file;
+    nautilus_files_view_class->add_files = nautilus_canvas_view_add_files;
     nautilus_files_view_class->begin_loading = nautilus_canvas_view_begin_loading;
     nautilus_files_view_class->bump_zoom_level = nautilus_canvas_view_bump_zoom_level;
     nautilus_files_view_class->can_zoom_in = nautilus_canvas_view_can_zoom_in;
@@ -2017,7 +2040,6 @@ nautilus_canvas_view_class_init (NautilusCanvasViewClass *klass)
     nautilus_files_view_class->get_view_id = nautilus_canvas_view_get_id;
     nautilus_files_view_class->get_first_visible_file = canvas_view_get_first_visible_file;
     nautilus_files_view_class->scroll_to_file = canvas_view_scroll_to_file;
-    nautilus_files_view_class->get_icon = nautilus_canvas_view_get_icon;
 
     properties[PROP_SUPPORTS_AUTO_LAYOUT] =
         g_param_spec_boolean ("supports-auto-layout",
@@ -2054,14 +2076,15 @@ nautilus_canvas_view_class_init (NautilusCanvasViewClass *klass)
 static void
 nautilus_canvas_view_init (NautilusCanvasView *canvas_view)
 {
+    NautilusCanvasViewPrivate *priv;
     NautilusCanvasContainer *canvas_container;
     GActionGroup *view_action_group;
     GtkClipboard *clipboard;
 
-    canvas_view->details = g_new0 (NautilusCanvasViewDetails, 1);
-    canvas_view->details->sort = &sort_criteria[0];
-    canvas_view->details->icon = g_themed_icon_new ("view-grid-symbolic");
-    canvas_view->details->destroyed = FALSE;
+    priv = nautilus_canvas_view_get_instance_private (canvas_view);
+
+    priv->sort = &sort_criteria[0];
+    priv->destroyed = FALSE;
 
     canvas_container = create_canvas_container (canvas_view);
     initialize_canvas_container (canvas_view, canvas_container);
