@@ -32,34 +32,17 @@
 #include "nautilus-file.h"
 #include "nautilus-link.h"
 
-typedef struct NautilusFilesView NautilusFilesView;
-typedef struct NautilusFilesViewClass NautilusFilesViewClass;
-
 #include "nautilus-window.h"
 #include "nautilus-view.h"
 #include "nautilus-window-slot.h"
 
+G_BEGIN_DECLS
+
 #define NAUTILUS_TYPE_FILES_VIEW nautilus_files_view_get_type()
-#define NAUTILUS_FILES_VIEW(obj)\
-        (G_TYPE_CHECK_INSTANCE_CAST ((obj), NAUTILUS_TYPE_FILES_VIEW, NautilusFilesView))
-#define NAUTILUS_FILES_VIEW_CLASS(klass)\
-        (G_TYPE_CHECK_CLASS_CAST ((klass), NAUTILUS_TYPE_FILES_VIEW, NautilusFilesViewClass))
-#define NAUTILUS_IS_FILES_VIEW(obj)\
-        (G_TYPE_CHECK_INSTANCE_TYPE ((obj), NAUTILUS_TYPE_FILES_VIEW))
-#define NAUTILUS_IS_FILES_VIEW_CLASS(klass)\
-        (G_TYPE_CHECK_CLASS_TYPE ((klass), NAUTILUS_TYPE_FILES_VIEW))
-#define NAUTILUS_FILES_VIEW_GET_CLASS(obj)\
-        (G_TYPE_INSTANCE_GET_CLASS ((obj), NAUTILUS_TYPE_FILES_VIEW, NautilusFilesViewClass))
 
-typedef struct NautilusFilesViewDetails NautilusFilesViewDetails;
+G_DECLARE_DERIVABLE_TYPE (NautilusFilesView, nautilus_files_view, NAUTILUS, FILES_VIEW, GtkGrid)
 
-struct NautilusFilesView {
-        GtkGrid parent;
-
-        NautilusFilesViewDetails *details;
-};
-
-struct NautilusFilesViewClass {
+struct _NautilusFilesViewClass {
         GtkGridClass parent_class;
 
         /* The 'clear' signal is emitted to empty the view of its contents.
@@ -74,12 +57,12 @@ struct NautilusFilesViewClass {
          */
         void         (* begin_file_changes)     (NautilusFilesView *view);
 
-        /* The 'add_file' signal is emitted to add one file to the view.
+        /* The 'add_files' signal is emitted to add a set of files to the view.
          * It must be replaced by each subclass.
          */
-        void    (* add_file)                    (NautilusFilesView *view,
-                                                 NautilusFile      *file,
-                                                 NautilusDirectory *directory);
+        void    (* add_files)                    (NautilusFilesView *view,
+                                                  GList             *files,
+                                                  NautilusDirectory *directory);
         void    (* remove_file)                 (NautilusFilesView *view,
                                                  NautilusFile      *file,
                                                  NautilusDirectory *directory);
@@ -261,9 +244,6 @@ struct NautilusFilesViewClass {
         void           (* check_empty_states)          (NautilusFilesView *view);
 };
 
-/* GObject support */
-GType               nautilus_files_view_get_type                         (void);
-
 NautilusFilesView *      nautilus_files_view_new                         (guint               id,
                                                                           NautilusWindowSlot *slot);
 
@@ -344,8 +324,6 @@ char *            nautilus_files_view_get_title                  (NautilusFilesV
 gboolean          nautilus_files_view_supports_zooming           (NautilusFilesView      *view);
 void              nautilus_files_view_bump_zoom_level            (NautilusFilesView      *view,
                                                                   int                     zoom_increment);
-void              nautilus_files_view_zoom_to_level              (NautilusFilesView      *view,
-                                                                  gint                    level);
 gboolean          nautilus_files_view_can_zoom_in                (NautilusFilesView      *view);
 gboolean          nautilus_files_view_can_zoom_out               (NautilusFilesView      *view);
 void              nautilus_files_view_update_menus               (NautilusFilesView      *view);
@@ -360,4 +338,6 @@ void              nautilus_files_view_action_show_hidden_files   (NautilusFilesV
 GActionGroup *    nautilus_files_view_get_action_group           (NautilusFilesView      *view);
 GtkWidget*        nautilus_files_view_get_content_widget         (NautilusFilesView      *view);
 
+G_END_DECLS
+
 #endif /* NAUTILUS_FILES_VIEW_H */
