@@ -169,41 +169,19 @@ nautilus_gmenu_add_item_in_submodel (GMenu       *menu,
 void
 nautilus_pop_up_context_menu (GtkWidget      *parent,
                               GMenu          *menu,
-                              GdkEventButton *event)
+                              GdkEventButton *button_event)
 {
     GtkWidget *gtk_menu;
 
-    int button;
-
     g_return_if_fail (G_IS_MENU (menu));
     g_return_if_fail (GTK_IS_WIDGET (parent));
 
     gtk_menu = gtk_menu_new_from_model (G_MENU_MODEL (menu));
     gtk_menu_attach_to_widget (GTK_MENU (gtk_menu), parent, NULL);
 
-    /* The event button needs to be 0 if we're popping up this menu from
-     * a button release, else a 2nd click outside the menu with any button
-     * other than the one that invoked the menu will be ignored (instead
-     * of dismissing the menu). This is a subtle fragility of the GTK menu code.
-     */
-    if (event)
-    {
-        button = event->type == GDK_BUTTON_RELEASE
-                 ? 0
-                 : event->button;
-    }
-    else
-    {
-        button = 0;
-    }
-
-    gtk_menu_popup (GTK_MENU (gtk_menu),                        /* menu */
-                    NULL,                                       /* parent_menu_shell */
-                    NULL,                                       /* parent_menu_item */
-                    NULL,                                       /* popup_position_func */
-                    NULL,                                       /* popup_position_data */
-                    button,                                     /* button */
-                    event ? event->time : gtk_get_current_event_time ());     /* activate_time */
+    gtk_menu_popup_at_pointer (GTK_MENU (gtk_menu),
+                               button_event ? (GdkEvent *) button_event :
+                               gtk_get_current_event ());
 
     g_object_ref_sink (gtk_menu);
     g_object_unref (gtk_menu);
@@ -391,32 +369,40 @@ nautilus_file_date_in_between (guint64    unix_file_time,
 }
 
 static const gchar *
-get_text_for_days_ago (gint days)
+get_text_for_days_ago (gint     days,
+                       gboolean prefix_with_since)
 {
     if (days < 7)
     {
         /* days */
-        return ngettext ("%d day ago", "%d days ago", days);
+        return prefix_with_since ?
+               ngettext ("Since %d day ago", "Since %d days ago", days) :
+               ngettext ("%d day ago", "%d days ago", days);
     }
-    else if (days < 30)
+    if (days < 30)
     {
         /* weeks */
-        return ngettext ("Last week", "%d weeks ago", days / 7);
+        return prefix_with_since ?
+               ngettext ("Since last week", "Since %d weeks ago", days / 7) :
+               ngettext ("Last week", "%d weeks ago", days / 7);
     }
-    else if (days < 365)
+    if (days < 365)
     {
         /* months */
-        return ngettext ("Last month", "%d months ago", days / 30);
-    }
-    else
-    {
-        /* years */
-        return ngettext ("Last year", "%d years ago", days / 365);
+        return prefix_with_since ?
+               ngettext ("Since last month", "Since %d months ago", days / 30) :
+               ngettext ("Last month", "%d months ago", days / 30);
     }
+
+    /* years */
+    return prefix_with_since ?
+           ngettext ("Since last year", "Since %d years ago", days / 365) :
+           ngettext ("Last year", "%d years ago", days / 365);
 }
 
 gchar *
-get_text_for_date_range (GPtrArray *date_range)
+get_text_for_date_range (GPtrArray *date_range,
+                         gboolean   prefix_with_since)
 {
     gint days;
     gint normalized;
@@ -464,7 +450,9 @@ get_text_for_date_range (GPtrArray *date_range)
 
 #pragma GCC diagnostic push
 #pragma GCC diagnostic ignored "-Wformat-nonliteral"
-        label = g_strdup_printf (get_text_for_days_ago (days), normalized);
+        label = g_strdup_printf (get_text_for_days_ago (days,
+                                                        prefix_with_since),
+                                 normalized);
 #pragma GCC diagnostic pop
     }
 
