@@ -28,7 +28,9 @@
 #include "nautilus-list-view.h"
 #include "nautilus-mime-actions.h"
 #include "nautilus-special-location-bar.h"
+#include "nautilus-toolbar.h"
 #include "nautilus-trash-bar.h"
+#include "nautilus-trash-monitor.h"
 #include "nautilus-view.h"
 #include "nautilus-window.h"
 #include "nautilus-x-content-bar.h"
@@ -147,6 +149,10 @@ static void nautilus_window_slot_set_search_visible (NautilusWindowSlot *self,
 static gboolean nautilus_window_slot_get_search_visible (NautilusWindowSlot *self);
 static void nautilus_window_slot_set_location (NautilusWindowSlot *self,
                                                GFile              *location);
+static void trash_state_changed_cb (NautilusTrashMonitor *monitor,
+                                    gboolean              is_empty,
+                                    gpointer              user_data);
+
 gboolean
 nautilus_window_slot_handles_location (NautilusWindowSlot *self,
                                        GFile              *location)
@@ -886,6 +892,10 @@ nautilus_window_slot_init (NautilusWindowSlot *self)
     priv = nautilus_window_slot_get_instance_private (self);
     app = g_application_get_default ();
 
+    g_signal_connect (nautilus_trash_monitor_get (),
+                      "trash-state-changed",
+                      G_CALLBACK (trash_state_changed_cb), self);
+
     priv->slot_action_group = G_ACTION_GROUP (g_simple_action_group_new ());
     g_action_map_add_action_entries (G_ACTION_MAP (priv->slot_action_group),
                                      slot_entries,
@@ -1354,7 +1364,7 @@ nautilus_window_slot_display_view_selection_failure (NautilusWindow *window,
         }
         else
         {
-            detail_message = g_strdup (_("This location doesn't appear to be a folder."));
+            detail_message = g_strdup (_("This location doesn’t appear to be a folder."));
         }
     }
     else if (error->domain == G_IO_ERROR)
@@ -1362,60 +1372,60 @@ nautilus_window_slot_display_view_selection_failure (NautilusWindow *window,
         switch (error->code)
         {
             case G_IO_ERROR_NOT_FOUND:
-                {
-                    detail_message = g_strdup (_("Unable to find the requested file. Please check the spelling and try again."));
-                }
-                break;
+            {
+                detail_message = g_strdup (_("Unable to find the requested file. Please check the spelling and try again."));
+            }
+            break;
 
             case G_IO_ERROR_NOT_SUPPORTED:
+            {
+                scheme_string = g_file_get_uri_scheme (location);
+                if (scheme_string != NULL)
                 {
-                    scheme_string = g_file_get_uri_scheme (location);
-                    if (scheme_string != NULL)
-                    {
-                        detail_message = g_strdup_printf (_("“%s” locations are not supported."),
-                                                          scheme_string);
-                    }
-                    else
-                    {
-                        detail_message = g_strdup (_("Unable to handle this kind of location."));
-                    }
-                    g_free (scheme_string);
+                    detail_message = g_strdup_printf (_("“%s” locations are not supported."),
+                                                      scheme_string);
                 }
-                break;
+                else
+                {
+                    detail_message = g_strdup (_("Unable to handle this kind of location."));
+                }
+                g_free (scheme_string);
+            }
+            break;
 
             case G_IO_ERROR_NOT_MOUNTED:
-                {
-                    detail_message = g_strdup (_("Unable to access the requested location."));
-                }
-                break;
+            {
+                detail_message = g_strdup (_("Unable to access the requested location."));
+            }
+            break;
 
             case G_IO_ERROR_PERMISSION_DENIED:
-                {
-                    detail_message = g_strdup (_("Don't have permission to access the requested location."));
-                }
-                break;
+            {
+                detail_message = g_strdup (_("Don’t have permission to access the requested location."));
+            }
+            break;
 
             case G_IO_ERROR_HOST_NOT_FOUND:
-                {
-                    /* This case can be hit for user-typed strings like "foo" due to
-                     * the code that guesses web addresses when there's no initial "/".
-                     * But this case is also hit for legitimate web addresses when
-                     * the proxy is set up wrong.
-                     */
-                    detail_message = g_strdup (_("Unable to find the requested location. Please check the spelling or the network settings."));
-                }
-                break;
+            {
+                /* This case can be hit for user-typed strings like "foo" due to
+                 * the code that guesses web addresses when there's no initial "/".
+                 * But this case is also hit for legitimate web addresses when
+                 * the proxy is set up wrong.
+                 */
+                detail_message = g_strdup (_("Unable to find the requested location. Please check the spelling or the network settings."));
+            }
+            break;
 
             case G_IO_ERROR_CANCELLED:
             case G_IO_ERROR_FAILED_HANDLED:
-                {
-                    goto done;
-                }
+            {
+                goto done;
+            }
 
             default:
-                {
-                }
-                break;
+            {
+            }
+            break;
         }
     }
 
@@ -2191,22 +2201,22 @@ update_history (NautilusWindowSlot         *self,
     switch (type)
     {
         case NAUTILUS_LOCATION_CHANGE_STANDARD:
-            {
-                handle_go_elsewhere (self, new_location);
-                return;
-            }
+        {
+            handle_go_elsewhere (self, new_location);
+            return;
+        }
 
         case NAUTILUS_LOCATION_CHANGE_RELOAD:
-            {
-                /* for reload there is no work to do */
-                return;
-            }
+        {
+            /* for reload there is no work to do */
+            return;
+        }
 
         case NAUTILUS_LOCATION_CHANGE_BACK:
-            {
-                handle_go_direction (self, new_location, FALSE);
-                return;
-            }
+        {
+            handle_go_direction (self, new_location, FALSE);
+            return;
+        }
 
         case NAUTILUS_LOCATION_CHANGE_FORWARD:
             handle_go_direction (self, new_location, TRUE);
@@ -2308,6 +2318,30 @@ out:
     g_free (data);
 }
 
+static void
+trash_state_changed_cb (NautilusTrashMonitor *monitor,
+                        gboolean              is_empty,
+                        gpointer              user_data)
+{
+    GFile *location;
+    NautilusDirectory *directory;
+
+    location = nautilus_window_slot_get_current_location (user_data);
+
+    if (location == NULL)
+    {
+        return;
+    }
+
+    directory = nautilus_directory_get (location);
+
+    if (nautilus_directory_is_in_trash (directory) &&
+        nautilus_trash_monitor_is_empty ())
+    {
+        nautilus_window_slot_remove_extra_location_widgets (user_data);
+    }
+}
+
 static void
 nautilus_window_slot_show_trash_bar (NautilusWindowSlot *self)
 {
@@ -2403,7 +2437,13 @@ view_started_loading (NautilusWindowSlot *self,
         nautilus_window_slot_set_allow_stop (self, TRUE);
     }
 
-    gtk_widget_grab_focus (GTK_WIDGET (priv->window));
+    /* Only grab focus if the menu isn't showing. Otherwise the menu disappears
+     * e.g. when the user toggles Show Hidden Files
+     */
+    if (!nautilus_toolbar_is_menu_visible (NAUTILUS_TOOLBAR (nautilus_window_get_toolbar (priv->window))))
+    {
+        gtk_widget_grab_focus (GTK_WIDGET (priv->window));
+    }
 
     gtk_widget_show (GTK_WIDGET (priv->window));
 
@@ -2481,7 +2521,10 @@ nautilus_window_slot_setup_extra_location_widgets (NautilusWindowSlot *self)
 
     if (nautilus_directory_is_in_trash (directory))
     {
-        nautilus_window_slot_show_trash_bar (self);
+        if (!nautilus_trash_monitor_is_empty ())
+        {
+            nautilus_window_slot_show_trash_bar (self);
+        }
     }
     else
     {
@@ -2649,6 +2692,8 @@ nautilus_window_slot_dispose (GObject *object)
 
     nautilus_window_slot_remove_extra_location_widgets (self);
 
+    g_signal_handlers_disconnect_by_data (nautilus_trash_monitor_get (), self);
+
     if (priv->content_view)
     {
         widget = GTK_WIDGET (priv->content_view);
@@ -3057,16 +3102,19 @@ nautilus_window_slot_get_icon (NautilusWindowSlot *self)
             return nautilus_view_get_icon (NAUTILUS_VIEW_GRID_ID);
         }
         break;
+
         case NAUTILUS_VIEW_GRID_ID:
         {
             return nautilus_view_get_icon (NAUTILUS_VIEW_LIST_ID);
         }
         break;
+
         case NAUTILUS_VIEW_OTHER_LOCATIONS_ID:
         {
             return nautilus_view_get_icon (NAUTILUS_VIEW_OTHER_LOCATIONS_ID);
         }
         break;
+
         default:
         {
             return NULL;
