@@ -796,7 +796,8 @@ update_cursor (NautilusWindow *window)
 
     slot = nautilus_window_get_active_slot (window);
 
-    if (nautilus_window_slot_get_allow_stop (slot))
+    if (slot != NULL &&
+        nautilus_window_slot_get_allow_stop (slot))
     {
         GdkDisplay *display;
 
@@ -2280,16 +2281,16 @@ nautilus_window_set_property (GObject      *object,
     switch (arg_id)
     {
         case PROP_DISABLE_CHROME:
-            {
-                window->priv->disable_chrome = g_value_get_boolean (value);
-            }
-            break;
+        {
+            window->priv->disable_chrome = g_value_get_boolean (value);
+        }
+        break;
 
         default:
-            {
-                G_OBJECT_WARN_INVALID_PROPERTY_ID (object, arg_id, pspec);
-            }
-            break;
+        {
+            G_OBJECT_WARN_INVALID_PROPERTY_ID (object, arg_id, pspec);
+        }
+        break;
     }
 }
 
@@ -2306,10 +2307,10 @@ nautilus_window_get_property (GObject    *object,
     switch (arg_id)
     {
         case PROP_DISABLE_CHROME:
-            {
-                g_value_set_boolean (value, window->priv->disable_chrome);
-            }
-            break;
+        {
+            g_value_set_boolean (value, window->priv->disable_chrome);
+        }
+        break;
     }
 }
 
@@ -2916,7 +2917,7 @@ nautilus_window_show_about_dialog (NautilusWindow *window)
                             * box to give credit to the translator(s).
                             */
                            "translator-credits", _("translator-credits"),
-                           "logo-icon-name", "system-file-manager",
+                           "logo-icon-name", "org.gnome.Nautilus",
                            NULL);
 }
 
